"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const antlr = require("antlr4ts");
const nodes_1 = require("./nodes");
const TerminalNode_1 = require("antlr4ts/tree/TerminalNode");
function parse(lexerConstructor, parserConstructor, rootNode, code) {
    let inputStream = new antlr.ANTLRInputStream(code);
    let lexer = new lexerConstructor(inputStream);
    lexer.removeErrorListeners();
    lexer.addErrorListener({
        syntaxError(
            // tslint:disable-next-line no-any
            recognizer, offendingSymbol, line, charPositionInLine, msg, e) {
            throw new Error(msg + " at " + line + ":" + charPositionInLine);
        }
    });
    let tokenStream = new antlr.CommonTokenStream(lexer);
    let parser = new parserConstructor(tokenStream);
    parser.removeErrorListeners();
    parser.addErrorListener({
        syntaxError(
            // tslint:disable-next-line no-any
            recognizer, offendingSymbol, line, charPositionInLine, msg, e) {
            throw new Error(msg + " at " + line + ":" + charPositionInLine);
        }
    });
    let node = rootNode(parser);
    let tree = createSourceTree(node, code)[0];
    return tree;
}
exports.parse = parse;
function createSourceTree(antlrNode, fullCode, codePosition = {
        charIndex: 0
    }) {
    if (antlrNode instanceof antlr.ParserRuleContext) {
        let tree = new nodes_1.Tree(antlrNode.constructor.name);
        if (antlrNode.start.startIndex > codePosition.charIndex) {
            tree.children.push(new nodes_1.Space(fullCode.substring(codePosition.charIndex, antlrNode.start.startIndex)));
            codePosition.charIndex = antlrNode.start.startIndex;
        }
        for (let i = 0; i < antlrNode.childCount; i++) {
            let child = antlrNode.getChild(i);
            if (child instanceof antlr.ParserRuleContext ||
                child instanceof TerminalNode_1.TerminalNode) {
                tree.children.push(...createSourceTree(child, fullCode, codePosition));
            }
            else {
                throw new Error("Unexpected child: " + child);
            }
        }
        if (antlrNode.stop.stopIndex > codePosition.charIndex) {
            tree.children.push(new nodes_1.Space(fullCode.substring(codePosition.charIndex, antlrNode.stop.stopIndex + 1)));
            codePosition.charIndex = antlrNode.stop.stopIndex + 1;
        }
        return [tree];
    }
    else {
        let nodes = [];
        if (antlrNode.symbol.startIndex > codePosition.charIndex) {
            nodes.push(new nodes_1.Space(fullCode.substring(codePosition.charIndex, antlrNode.symbol.startIndex)));
            codePosition.charIndex = antlrNode.symbol.startIndex;
        }
        if (antlrNode.text !== "<EOF>") {
            nodes.push(new nodes_1.Token(antlrNode.symbol.type, antlrNode.text));
            codePosition.charIndex += antlrNode.text.length;
        }
        if (antlrNode.symbol.stopIndex > codePosition.charIndex) {
            nodes.push(new nodes_1.Space(fullCode.substring(codePosition.charIndex, antlrNode.symbol.stopIndex + 1)));
            codePosition.charIndex = antlrNode.symbol.stopIndex + 1;
        }
        return nodes;
    }
}
