"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const antlr = require("./antlr");
exports.antlr = antlr;
const nodes = require("./nodes");
exports.nodes = nodes;
const typescript = require("./typescript");
exports.typescript = typescript;
function printSource(node) {
    if (node instanceof nodes.Tree) {
        return node.children.map(childNode => printSource(childNode)).join("");
    }
    else if (node instanceof nodes.Token) {
        return node.text;
    }
    else {
        return node.text;
    }
}
exports.printSource = printSource;
function findTree(tree, searchPattern) {
    if (treesMatch(tree, searchPattern)) {
        return tree;
    }
    for (let child of tree.children) {
        if (child instanceof nodes.Tree) {
            let found = findTree(child, searchPattern);
            if (found) {
                return found;
            }
        }
    }
    return null;
}
exports.findTree = findTree;
function treesMatch(tree, searchPattern) {
    if (searchPattern instanceof nodes.Space) {
        throw new Error("Unexpected space in search searchPattern.");
    }
    if (getText(searchPattern) === "__") {
        return true;
    }
    if (!(searchPattern instanceof nodes.Tree)) {
        return false;
    }
    let treeChildrenWithoutSpaces = tree.children.filter(child => !(child instanceof nodes.Space));
    let searchPatternChildrenWithoutSpaces = searchPattern.children.filter(child => !(child instanceof nodes.Space));
    if (tree.nodeType !== searchPattern.nodeType ||
        treeChildrenWithoutSpaces.length !==
            searchPatternChildrenWithoutSpaces.length) {
        // TODO: Consider being more flexible, e.g. a function call with a variable number of arguments will not match
        // unless it has the exact same number of arguments.
        return false;
    }
    for (let i = 0; i < treeChildrenWithoutSpaces.length; i++) {
        let child = treeChildrenWithoutSpaces[i];
        let searchPatternChild = searchPatternChildrenWithoutSpaces[i];
        if (child instanceof nodes.Tree) {
            if (!treesMatch(child, searchPatternChild)) {
                return false;
            }
        }
        else if (child instanceof nodes.Token &&
            searchPatternChild instanceof nodes.Token) {
            if (child.tokenType !== searchPatternChild.tokenType) {
                return false;
            }
            if (searchPatternChild.text.indexOf("__") !== -1) {
                // Ignore text, match anything.
            }
            else if (child.text !== searchPatternChild.text) {
                return false;
            }
        }
        else {
            return false;
        }
    }
    return true;
}
exports.treesMatch = treesMatch;
function getText(node) {
    if (node instanceof nodes.Token) {
        return node.text;
    }
    else {
        return node.children
            .filter(childNode => !(childNode instanceof nodes.Space))
            .map(getText)
            .join("");
    }
}
function transformTree(tree, searchPattern, transform) {
    let cloned = cloneTree(tree);
    if (treesMatch(tree, searchPattern)) {
        return transform(cloned);
    }
    for (let i = 0; i < cloned.children.length; i++) {
        let child = cloned.children[i];
        if (child instanceof nodes.Tree) {
            cloned.children[i] = transformTree(child, searchPattern, transform);
        }
    }
    return cloned;
}
exports.transformTree = transformTree;
function cloneTree(tree) {
    let cloned = new nodes.Tree(tree.nodeType);
    cloned.children = tree.children.map(cloneNode);
    return cloned;
}
function cloneNode(node) {
    if (node instanceof nodes.Tree) {
        return cloneTree(node);
    }
    else if (node instanceof nodes.Token) {
        return new nodes.Token(node.tokenType, node.text);
    }
    else {
        return new nodes.Space(node.text);
    }
}
